
import java.util.Optional;
import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonBar.ButtonData;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Dialog;
import javafx.scene.control.Label;
import javafx.scene.control.PasswordField;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.stage.Stage;
import javafx.util.Callback;

/**
 * A (sub-) class representing a dialog, i.e. a temporary window used for e.g.
 * displaying a form. A dialog is modal by default; a call to dialog.showAndWait
 * also makes it blocking (i.e. the user can only interact with the dialog (not
 * the parent window) and the execution of the code showing the dialog waits,
 * until the dialog is closed).
 *
 * The way to transfer data between the dialog and the application is rather
 * awkward, see line 60-73 and the API docs at
 * https://docs.oracle.com/javase/8/javafx/api/javafx/scene/control/Dialog.html
 *
 * NB! super class Dialog is generic, hence the type parameter below.
 */
class LogOnDialog extends Dialog<UserInfo> {

    private TextField userNameField;
    private PasswordField passwordField;

    LogOnDialog() {
        super(); // super constructor, modal by default

        userNameField = new TextField();
        passwordField = new PasswordField();

        GridPane grid = new GridPane();
        grid.setVgap(5);
        grid.setHgap(5);
        grid.setPadding(new Insets(10, 10, 10, 10));
        grid.add(new Label("User name"), 1, 1);
        grid.add(userNameField, 2, 1);
        grid.add(new Label("Password"), 1, 2);
        grid.add(passwordField, 2, 2);
        this.getDialogPane().setContent(grid);

        ButtonType buttonTypeOk = new ButtonType(
                "Log on", ButtonData.OK_DONE);
        this.getDialogPane().getButtonTypes().add(buttonTypeOk);

        this.setResultConverter(new Callback<ButtonType, UserInfo>() {
            @Override
            public UserInfo call(ButtonType b) {
                if (b == buttonTypeOk) {

                    UserInfo info = new UserInfo(
                            userNameField.getText(),
                            passwordField.getText());
                    LogOnDialog.this.clearFields();
                    return info;
                }
                return null;
            }
        });

        // If input in the form is not valid, consume the event (without
        // executing method "call".
        final Button okButton = 
                (Button) this.getDialogPane().lookupButton(buttonTypeOk);
        okButton.addEventFilter(ActionEvent.ACTION, ae -> {
            if (!isValid()) {
                ae.consume(); //not valid
                showAlert("Invalid input!");
            }
        });
    }

    // Check whether form data is valid or not
    private boolean isValid() {
        // Check if input is valid (logic below is just an example)
        return userNameField.getText().length() >= 2
                && passwordField.getText().length() >= 6;
    }

    private void clearFields() {
        userNameField.setText("");
        passwordField.setText("");
    }

    void showAlert(String message) {
        Alert alert = new Alert(AlertType.WARNING, message);
        alert.showAndWait();
    }
}

/**
 * A simple representation of some data, to be "transferred" from the dialog to
 * the application when the call dialog.ShowAndWait() returns.
 */
class UserInfo {

    private final String name;
    private final String password;

    UserInfo(String name, String password) {
        this.name = name;
        this.password = password;
    }

    String getName() {
        return name;
    }

    String getPassword() {
        return password;
    }
}

/**
 * The application, implementing the main window (stage) and an event handler
 * calling dialog.showAndWait(), line 132-148, a blocking call.
 */
public class JavaFXDialogExample extends Application {

    private LogOnDialog logOnDialog;

    @Override
    public void start(Stage primaryStage) {

        logOnDialog = new LogOnDialog(); // modal by default

        VBox root = new VBox(20);
        root.setAlignment(Pos.CENTER);
        createApplicationUI(root);

        Scene scene = new Scene(root, 400, 200);

        primaryStage.setTitle("Hello World!");
        primaryStage.setScene(scene);
        primaryStage.show();
    }

    private void createApplicationUI(VBox root) {
        final Text infoText = new Text("- - -");
        infoText.setFont(Font.font("Consolas", 14));

        Button logOnButton = new Button("Log on");
        logOnButton.setOnAction(new EventHandler<ActionEvent>() {

            @Override
            public void handle(ActionEvent event) {
                Optional<UserInfo> result
                        = logOnDialog.showAndWait(); // blocking
                if (result.isPresent()) {
                    UserInfo user = result.get();
                    infoText.setText(
                            "Your password is '"
                            + user.getPassword()
                            + "' - Wohahaha!");
                }
            }
        });

        root.getChildren().addAll(logOnButton, infoText);
    }

    public static void main(String[] args) {
        launch(args);
    }
}
